/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.crop.persistence.serialization;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.CropDefinitionManager;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.model.*;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Wytse Visser
 */
@ServiceProvider(service = CropDefinitionManager.class)
public class CropDefinitionManagerImpl extends AbstractEntityManager<CropDefinition> implements CropDefinitionManager {

    private static final String CROP_DEF_FOLDER = "CropDefinitions";
    private static final Logger LOGGER = Logger.getLogger(CropDefinitionManagerImpl.class.getName());

    @Override
    public Set<CropDefinition> findAll() {
        Set<CropDefinition> cropDefinitions = new HashSet<>();
        FileObject cropDefinitionFolder = getCropDefinitionFolder();

        for (FileObject cropDefinitionFile : cropDefinitionFolder.getChildren()) {
            try {
                CropDefinition cropDefinition = load(cropDefinitionFile);
                cropDefinition = find(cropDefinition.getURI());
                cropDefinitions.add(cropDefinition);
            } catch (ClassNotFoundException | IOException | NullPointerException ex) {
                LOGGER.log(Level.WARNING, "Could not load CropDefinition from file", ex.getMessage());
            }
        }

        return cropDefinitions;
    }

    @Override
    public void save(CropDefinition cropDefinition) {
        try {
            FileObject targetFile = getTargetFile(cropDefinition);
            try (ObjectOutputStream oos = new ObjectOutputStream(targetFile.getOutputStream())) {
                oos.writeObject(cropDefinition);
                addToCache(cropDefinition);
            }
        } catch (IOException e) {
            LOGGER.log(Level.WARNING, "Could not save CropDefinition {0}", e.getMessage());
        }
    }
    
    @Override
    protected CropDefinition load(AgroURI cropURI) {
        LOGGER.log(Level.FINEST, "Loading CropDefinition with id {0} from file", cropURI.getItemIdNumber());
        CropDefinition cropDefinition = null;
        try {
            FileObject targetFile = getTargetFile(cropURI.getItemIdNumber());
            cropDefinition = load(targetFile);
            cropDefinition = getCachedInstanceFor(cropDefinition);
        } catch (IOException | ClassNotFoundException ex) {
            LOGGER.log(Level.WARNING, "Could not load CropDefinition from file", ex.getMessage());
        }
        
        return reloadCrop(cropDefinition);
    }

    private CropDefinition load(FileObject cropDefinitionFile) throws IOException, ClassNotFoundException {
        try (ObjectInputStream ois = new ObjectInputStream(cropDefinitionFile.getInputStream())) {
            return reloadCrop((CropDefinition) ois.readObject());
        }
    }
    
    private CropDefinition reloadCrop(CropDefinition cropDefinition) {
        CropManager cropManager = Lookup.getDefault().lookup(CropManager.class);
        Crop crop = cropManager.find(cropDefinition.getCrop().getURI());
        
        cropDefinition.setCrop(crop);
        
        return cropDefinition;
    }

    private FileObject getTargetFile(String itemIdNumber) throws IOException {
        FileObject targetFile = getCropDefinitionFolder().getFileObject(itemIdNumber, CropDefinition.EXT);
        if (targetFile == null) {
            targetFile = getCropDefinitionFolder().createData(itemIdNumber, CropDefinition.EXT);
        }

        return targetFile;
    }

    private FileObject getTargetFile(CropDefinition cropDefinition) throws IOException {
        return getTargetFile(cropDefinition.getItemIdNumber());
    }

    protected FileObject getCropDefinitionFolder() {
        FileObject configRoot = FileUtil.getConfigRoot();
        FileObject cropDefinitionFolder = configRoot.getFileObject(CROP_DEF_FOLDER);
        if (cropDefinitionFolder == null) {
            try {
                cropDefinitionFolder = configRoot.createFolder(CROP_DEF_FOLDER);
            } catch (IOException ex) {
                LOGGER.log(Level.WARNING, "Unable to create crop definition folder in config root {0}", ex.getMessage());
            }
        }

        return cropDefinitionFolder;
    }

    @Override
    protected CropDefinition load(AgroURI uri, Project project) {
        return load(uri); // CropDefinitions are farm independent
    }

    @Override
    public Set<ItemIdType> getSupportedTypes() {
        return EnumSet.of(ItemIdType.CTN);
    }

    @Override
    public void remove(CropDefinition cropDefinition) {
        removeFromCache(cropDefinition);
        try {
            getTargetFile(cropDefinition).delete();
        } catch (IOException e) {
            LOGGER.log(Level.WARNING, "Could not remove CropDefinition {0}. {1}",
                    new Object[]{cropDefinition, e.getMessage()
                    });
        }
    }
}
